OPERATION_OK = "OK"
OPERATION_ERROR = "ERROR"

LOG_DIAGNOSTIC = "diag"
LOG_INFO = "info"
LOG_WARNING = "warn"
LOG_ERROR = "error"

LOG_LEVELS = [LOG_DIAGNOSTIC, LOG_INFO, LOG_WARNING, LOG_ERROR]

PERIODE_PATH = "/periode"
PULJE_PATH = "/pulje"
UTSENDING_PATH = "/utsending"
SKJEMA_PATH = "/skjemadata/skjema"
PREFILL_META_PATH = "/skjemadata/prefill-meta/skjema"
INSTANCE_PATH = "/altinn/instance"
INSTANTIATOR_RESEND_PATH = "/instantiator/resend"
INNKVITTERING_RESEND_PATH = "/innkvittering/resend"
PREFILL_API_STAT_PATH = "/public/prefill"